#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <linux/joystick.h>

#include "ptz_network.h"

int main(int argc, char *argv[])
{
struct js_event e;
struct website_info_t website_info;
char *joy_dev=0;
char command[128];
int currx,curry,currz;
int fd;
int r;
fd_set readset;
struct timeval tv;
int dx,dy,dz;

  if (argc<2)
  {
    printf("Usage: joystick [options] <website address>\n");
    printf("   -p <port>\n");
    printf("   -u <username:password>\n");
    printf("   -j <joystick device: /dev/input/js0>\n\n");
    exit(0);
  }

  memset(&website_info,0,sizeof(struct website_info_t));
  website_info.port=80;

  /* Read in command line arguments */

  for (r=1; r<argc; r++)
  {
    if (strcmp(argv[r],"-p")==0)
    { website_info.port=atoi(argv[++r]); }
      else
    if (strcmp(argv[r],"-u")==0)
    { website_info.username_password=base64_encode(argv[++r]); }
      else
    if (strcmp(argv[r],"-j")==0)
    { joy_dev=argv[++r]; }
      else
    {
      website_info.host=argv[r];
    }
  }

  /* Open joystick /dev */

  if (joy_dev==0)
  { fd=open("/dev/input/js0",O_RDONLY); }
    else
  { fd=open(joy_dev,O_RDONLY); }

  if (fd==0)
  {
    printf("Could no open joystick device\n");
    exit(1);
  }

  /* Setup non-blocking I/O */

  fcntl(fd,F_SETFL,O_NONBLOCK);

  init_socket();

  currx=0;
  curry=0;
  currz=0;
  dx=0;
  dy=0;
  dz=0;

  /* Main loop for reading the joystick device.  Use non-blocking I/O
     to ignore many joystick changes in a row. */

  while(1)
  {
    tv.tv_sec=10;
    tv.tv_usec=0;

    FD_ZERO(&readset);
    FD_SET(fd,&readset);

    if ((r=select(fd+1,&readset,NULL,NULL,&tv))==-1)
    {
      printf("Select error\n");
      break;
    }

    if (r==0) continue;

    /* read joystick events and wait for things to settle down 
       before actually sending PTZ commands.  dx,dy,dz are the
       current changed values of the joystick.  After there are
       no more joystick events queued up, send events to the camera
       based on if they are not equal to the current x,y,z. */

    while(1)
    {
      r=read (fd, &e, sizeof(struct js_event));
      if (r==-1) break;

      if (e.type==JS_EVENT_AXIS)
      {
        r=((e.value*25/32768)*4);

        if (e.number==0)  /* Pan */
        { dx=r; }
          else
        if (e.number==1)  /* Tilt */
        { dy=-r; }
          else
        if (e.number==3)  /* Zoom */
        { dz=-r; }
      }
      usleep(100);
    }


#ifdef DEBUG
/*
    printf("  time: %d\n",e.time);
    printf("Event-----------\n");
    printf("  time: %d\n",e.time);
    printf(" value: %d\n",e.value);
    printf("  type: %d\n",e.type);
    printf("number: %d\n",e.number);
*/
#endif

    /* Make a web connection to the camera to move it.  These URLs can
       be changed for other brand of cameras.  The following code works
       with Axis cameras. */

    if (currx!=dx || curry!=dy)
    {
      currx=dx;
      curry=dy;
      sprintf(command,"/axis-cgi/com/ptz.cgi?continuouspantiltmove=%d,%d",currx,curry);
      send_command(&website_info,command);

#ifdef DEBUG
printf("Command: %s\n",command);
#endif
    }

    if (currz!=dz)
    {
      currz=dz;
      sprintf(command,"/axis-cgi/com/ptz.cgi?continuouszoommove=%d",currz);
      send_command(&website_info,command);

#ifdef DEBUG
printf("Command: %s\n",command);
#endif
    }
  }

  close_socket(&website_info);
  close(fd);

  return 0; 
}

