#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifndef WINDOWS
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>
#endif
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#ifdef WINDOWS
#include <windows.h>
#include <winsock.h>
#endif

#include "ptz_network.h"

void setsignals();

void broken_pipe()
{
  setsignals();
}

void other_signal()
{
  setsignals();
}

void setsignals()
{
  signal(SIGPIPE,broken_pipe);
  signal(SIGURG,other_signal);
  signal(SIGIO,other_signal);
  signal(SIGHUP,other_signal);

/*
  signal(SIGINT,destroy);
  signal(SIGTERM,destroy);
*/
}

/* init_socket() basically just sets stuff up, like the Unix signals
   incase of a broken pipe and sets up Windows Winsock retardacies.
   Even tho it won't compile on Windows due to the joystick code, it
   could be modified for it later.  All the socket code should compile
   on Windows too. */

int init_socket()
{
#ifdef WINDOWS
WSADATA wsaData;
WORD wVersionRequested=MAKEWORD(1,1);
int Win32isStupid;

  Win32isStupid=WSAStartup(wVersionRequested, &wsaData);
  if (Win32isStupid)  /* Shouldn't this always be true? :) */
  {
    printf("Winsock can't start.\n");
    exit(1);
  }
#else
  setsignals();
#endif

  return 0;
}

/* send_request() sends a tcp message to the web server using select()
   to make sure it's okay to send() */

int send_request(int socketid, char *message)
{
struct timeval tv;
fd_set writeset;
int t;

  FD_ZERO(&writeset);
  FD_SET(socketid,&writeset);

  tv.tv_sec=2;
  tv.tv_usec=0;

  if (((t=select(socketid+1,NULL,&writeset,NULL,&tv))==-1) && (errno!=EINTR))
  {
    printf("Problem with select\n");
    return -1;
  }
    else
  {
    if (send(socketid,message,strlen(message),0)<=0) return -1;
  }

  return 0;
}

/* read_buffer() reads from a tcp socket a single string ending on
   a \r or \r\n.  This code isn't very good actually and needs to
   be re-written. */

int read_buffer(struct website_info_t *website_info)
{
fd_set readset;
struct timeval tv;
int r,t=0;
int tries;

  tries=0;

  while(1)
  {
    errno=0;
    if (website_info->ptr>=website_info->len)
    {
      tv.tv_sec=10;
      tv.tv_usec=0;

      FD_ZERO(&readset);
      FD_SET(website_info->socketid,&readset);
      if (((t=select(website_info->socketid+1,&readset,NULL,NULL,&tv))==-1) && (errno!=EINTR))
      { return -1; }

      if (FD_ISSET(website_info->socketid,&readset))
      {  }
        else
      {
        tries++;
        if (tries==5)
        {
          printf("Problem reading from socket.\n");
          return -1;
        }
        continue;
      }

      tries=0;
      website_info->len=recv(website_info->socketid,(char *)website_info->socket_buffer,512,0);

      website_info->ptr=0;
      if (website_info->len>=0) website_info->socket_buffer[website_info->len]=0;

      if (website_info->len==0) return -1;
      if (website_info->len<0)
      {
        website_info->buffer_ptr=0;
        website_info->len=0;
        return -1;
      }
    }

    for (r=website_info->ptr; r<website_info->len; r++)
    {
      if ((website_info->socket_buffer[r]=='\n' || website_info->socket_buffer[r]=='\r') || website_info->socket_buffer[r]==0)
      {
        t=-1;
        break;
      }

      if (website_info->socket_buffer[r]==127 || website_info->socket_buffer[r]==8)
      { if (website_info->buffer_ptr>0) website_info->buffer_ptr--; }
        else
      { if (website_info->socket_buffer[r]>=32 && website_info->socket_buffer[r]<255) website_info->buffer[website_info->buffer_ptr++]=website_info->socket_buffer[r]; }

      t=1;
    }

    website_info->ptr=r+1;
    website_info->buffer[website_info->buffer_ptr]=0;

    if ((t==-1 && website_info->buffer_ptr>0) || website_info->buffer_ptr>512)
    {
      website_info->buffer_ptr=0;
      if (website_info->socket_buffer[website_info->ptr]=='\n' || website_info->socket_buffer[website_info->ptr]=='\r')
      { website_info->ptr++; }

      return 1;
    }
  }
}

/* connect_server() opens the connection to the webserver/IP camera */

int connect_server(struct website_info_t *website_info)
{
struct sockaddr_in serv_addr;
struct hostent *hp;

  website_info->buffer_ptr=0;
  website_info->len=0;
  website_info->ptr=0;

  if ((hp=gethostbyname(website_info->host))==0)
  {
    printf("Host lookup failed %s.\n",website_info->host);
    return -1;
  }

  memset((char*)&serv_addr,0,sizeof(serv_addr)); 
  serv_addr.sin_family=AF_INET;
  serv_addr.sin_port=htons(website_info->port);
  memcpy((char *)&serv_addr.sin_addr,hp->h_addr_list[0],hp->h_length);

  if ((website_info->socketid=socket(AF_INET,SOCK_STREAM,0))<0)
  {
    website_info->socketid=0;
    printf("Could not open socket.\n");
    return -1;
  }

  if (connect(website_info->socketid,(struct sockaddr *) &serv_addr,sizeof(serv_addr))<0)
  {
    close(website_info->socketid);
    website_info->socketid=0;
    printf("Could not connect to server.\n");
    return -1;
  }

  return 0;
}

/* send_command() calls connect_server() if needed and sends the
   request to the webserver of the IP camera with a command.  It
   will try to do a keep-alive to make sure the socket doesn't close
   after the request is made, but unfortunately not all IP cameras
   support this. */

int send_command(struct website_info_t *website_info, char *page)
{
char buffer[1024];
int header_flag=0;
int keepalive=1;
int tries;
int i;

  if (website_info->username_password==0)
  {
    sprintf(buffer,"GET %s HTTP/1.1\r\n"
                   "Host: %s:%d\r\n"
                   "Connection: Keep-Alive\r\n\r\n",page,website_info->host,website_info->port);
  }
    else
  {
    sprintf(buffer,"GET %s HTTP/1.1\r\n"
                   "Host: %s:%d\r\n"
                   "Authorization: Basic %s\r\n"
                   "Connection: Keep-Alive\r\n\r\n",page,website_info->host,website_info->port,website_info->username_password);
  }

  tries=0;

  while(tries<3)
  {
    if (website_info->socketid==0)
    {
printf("connecting to %s\n",website_info->host);
      connect_server(website_info);
    }

    if (send_request(website_info->socketid,buffer)<0)
    {
      close(website_info->socketid);
      website_info->socketid=0;
    }
      else
    { break; }

    tries++;
  }

printf("tries=%d\n",tries);
  if (tries==3) return -1;

  header_flag=0;

  while(1)
  {
    i=read_buffer(website_info);
printf("%d> buffer=%s\n",header_flag,website_info->buffer);
    if (i<0) { header_flag++; }
    if (header_flag>1) break;

    /* FIXME - this line is pretty bad */
    if (strcasestr(website_info->buffer,"204 No Content")!=0)
    { header_flag++; keepalive=0; break; }
      else
    if (strcasestr(website_info->buffer,"Connection: Close")==0) keepalive=0;
  }

  if (keepalive==0)
  {
    close(website_info->socketid);
    website_info->socketid=0; 
  }

  return 0;
}

/* close_socket() closes the connection. */

void close_socket(struct website_info_t *website_info)
{
  if (website_info->username_password!=0)
  {
    free(website_info->username_password);
    website_info->username_password=0;
  }

  close(website_info->socketid);
}

/* base64_encode() is used for server authentication.  Username:password
   is base64 encoded and sent in the Authorization header */

char *base64_encode(char *text_in)
{
int t,l,ptr;
int holding,bitptr;
char *user_pass_64;
char base64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZ"
              "abcdefghijklmnopqrstuvwxyz"
              "0123456789+/";

  l=strlen(text_in);
  user_pass_64=(char *)malloc(((l*4)/3)+5);
  t=0;

  ptr=0;

  holding=0;
  bitptr=0;

  while(t<l)
  {
    if (bitptr<6)
    {
      holding=(holding<<8)+text_in[t++];
      bitptr=bitptr+8;
    }

    bitptr=bitptr-6;
    user_pass_64[ptr++]=base64[holding>>bitptr];
    holding=holding&((1<<bitptr)-1);
  }

  while (bitptr>0)
  {
    if (bitptr>=6)
    { user_pass_64[ptr++]=base64[holding>>(bitptr-6)]; }
      else
    { user_pass_64[ptr++]=base64[holding<<(6-bitptr)]; }

    bitptr=bitptr-6;
    holding=holding&((1<<bitptr)-1);
  }

  if ((ptr%4)!=0)
  {
    l=4-(ptr%4);

    for (t=0; t<l; t++)
    { user_pass_64[ptr++]='='; }
  }

  user_pass_64[ptr++]=0;

  return user_pass_64;
}


